@@ -6,7 +6,6 @@ import importlib
 import importlib.util
 from copy import deepcopy
 from pathlib import Path
-import yaml
 from typing import Union
 
 from steam_sdk.data.DataSettings import DataSettings
@@ -16,7 +15,9 @@ from steam_sdk.drivers.DriverFiQuS import DriverFiQuS
 from steam_sdk.drivers.DriverLEDET import DriverLEDET
 from steam_sdk.drivers.DriverPSPICE import DriverPSPICE
 from steam_sdk.drivers.DriverPyBBQ import DriverPyBBQ
-from steam_sdk.parsers.ParserYAML import dict_to_yaml, yaml_to_data_class
+from steam_sdk.drivers.DriverXYCE import DriverXYCE
+from steam_sdk.parsers.ParserYAML import dict_to_yaml, yaml_to_data
+from steam_sdk.postprocs.PostprocsMetrics import PostprocsMetrics
 from steam_sdk.utils.make_folder_if_not_existing import make_folder_if_not_existing
 from steam_sdk.utils.rgetattr import rgetattr
 from steam_sdk.utils.sgetattr import rsetattr
@@ -28,41 +29,33 @@ class AnalysisSTEAM:
         Class to run analysis based on STEAM_SDK
     """
 
-    def __init__(self, file_name_analysis: str = None,
+    def __init__(self, file_name_analysis: str,
                  relative_path_settings: str = '',
                  verbose: bool = False):
         """
-            Analysis based on STEAM_SDK
+        Analysis based on STEAM_SDK
+        :param file_name_analysis: full path to analysis.yaml input file  # object containing the information read from the analysis input file
+        :param relative_path_settings: relative path to settings.xxx.yaml file
+        :param verbose: if true, more information is printed to the console
         """
 
         # Initialize
-        self.file_name_analysis = file_name_analysis
-        self.relative_path_settings = Path(relative_path_settings).resolve()
+        self.data_analysis = yaml_to_data(file_name_analysis, DataAnalysis)  # Load yaml keys into DataAnalysis dataclass
+        self.settings = DataSettings()  # object containing the settings acquired during initialization
+        self.library_path = None
+        self.output_path = None
+        self.temp_path = None
+        self.list_models = {}  # this dictionary will be populated with BuilderModel objects and their names
+        self.list_sims = []  # this list will be populated with integers indicating simulations to run
+        self.list_viewers = {}  # this dictionary will be populated with Viewer objects and their names
+        self.list_metrics = {}  # this dictionary will be populated with calculated metrics
         self.verbose = verbose
-        self.data_analysis = None  # object containing the information read from the analysis input file
-        self.settings      = DataSettings()  # object containing the settings acquired during initialization
-        self.library_path  = None
-        self.output_path   = None
-        self.temp_path     = None
-        self.list_models   = {}  # this dictionary will be populated with BuilderModel objects and their names
-        self.list_sims     = []  # this list will be populated with integers indicating simulations to run
-        self.list_viewers  = {}  # this dictionary will be populated with Viewer objects and their names
-
-        # Check that input file is provided
-        if not self.file_name_analysis:
-            raise Exception('No .yaml input file provided.')
-
-        # Load yaml keys into DataAnalysis dataclass
-        self.data_analysis = yaml_to_data_class(self.file_name_analysis, DataAnalysis)
-        # with open(self.file_name_analysis, "r") as stream:
-        #     dictionary_yaml = yaml.safe_load(stream)
-        #     self.data_analysis = DataAnalysis(**dictionary_yaml)
 
         # Read working folders and set them up
-        self.set_up_working_folders()
+        self._set_up_working_folders()
 
         # Read analysis settings
-        self.read_settings()
+        self._load_and_write_settings(relative_path_settings)
 
     def setAttribute(self, dataclassSTEAM, attribute: str, value):
         try:
@@ -76,7 +69,7 @@ class AnalysisSTEAM:
         except:
             return getattr(getattr(self, dataclassSTEAM), attribute)
 
-    def set_up_working_folders(self):
+    def _set_up_working_folders(self):
         """
             ** Read working folders and set them up **
             This method performs the following tasks:
@@ -86,93 +79,79 @@ class AnalysisSTEAM:
              - Check if temporary folder is present. If so, delete it.
         """
 
-        # Unpack inputs
-        verbose = self.verbose
-        library_path = self.data_analysis.WorkingFolders.library_path
-        output_path = self.data_analysis.WorkingFolders.output_path
-        temp_path = self.data_analysis.WorkingFolders.temp_path
-
         # Raise exceptions if folders are not defined.
-        if not library_path:
+        if not self.data_analysis.WorkingFolders.library_path:
             raise Exception('Model library path must be defined. Key to provide: WorkingFolders.library_path')
-        if not output_path:
+        if not self.data_analysis.WorkingFolders.output_path:
             raise Exception('Output folder path must be defined. Key to provide: WorkingFolders.output_path')
-        if not temp_path:
+        if not self.data_analysis.WorkingFolders.temp_path:
             raise Exception('Temporary folder path must be defined. Key to provide: WorkingFolders.temp_path')
 
         # Resolve all paths and re-assign them to the self variables
+
         self.library_path = Path(self.data_analysis.WorkingFolders.library_path).resolve()
         self.output_path = Path(self.data_analysis.WorkingFolders.output_path).resolve()
         self.temp_path = Path(self.data_analysis.WorkingFolders.temp_path).resolve()
-        library_path = self.library_path
-        output_path = self.output_path
-        temp_path = self.temp_path
 
-        if verbose:
-            print('Model library path:    {}'.format(library_path))
-            print('Output folder path:    {}'.format(output_path))
-            print('Temporary folder path: {}'.format(temp_path))
+        if self.verbose:
+            print('Model library path:    {}'.format(self.library_path))
+            print('Output folder path:    {}'.format(self.output_path))
+            print('Temporary folder path: {}'.format(self.temp_path))
 
         # Check if model library folder is present. If not, raise an exception.
-        if not os.path.isdir(library_path):
-            raise Exception(
-                'Model library path refers to a not-existing folder: {}. Key to change: WorkingFolders.library_path'.format(
-                    library_path))
+        if not os.path.isdir(self.library_path):
+            raise Exception(f'Model library path refers to a not-existing folder: {self.library_path}. Key to change: WorkingFolders.library_path')
 
         # Check if output folder is present. If not, make it.
-        if not os.path.isdir(output_path):
-            Path(output_path).mkdir(parents=True, exist_ok=True)
-            if self.verbose: print('Folder {} did not exist. It was made now.'.format(output_path))
+        make_folder_if_not_existing(self.output_path, verbose=self.verbose)
 
         # Check if temporary folder is present. If so, delete it.
-        if os.path.isdir(temp_path):
-            shutil.rmtree(temp_path)
-            if self.verbose: print('Folder {} already existed. It was removed.'.format(temp_path))
+        if os.path.isdir(self.temp_path):
+            shutil.rmtree(self.temp_path)
+            if self.verbose: print('Folder {} already existed. It was removed.'.format(self.temp_path))
 
-    def read_settings(self):
+    def _load_and_write_settings(self, relative_path_settings: str):
         """
             ** Read analysis settings **
-
             They will be read either form a local settings file (if flag_permanent_settings=False)
             or from the keys in the input analysis file (if flag_permanent_settings=True)
+            :param relative_path_settings: only used if flag_permanent_settings=False and allows to specify folder containing settings.user_name.yaml
+            :rtype: nothing, saves temporary settings.user_name.yaml on disk
         """
 
         verbose = self.verbose
+        user_name = os.getlogin()
+        settings_file = f"settings.{user_name}.yaml"
 
         if self.data_analysis.GeneralParameters.flag_permanent_settings:
             # Read settings from analysis input file (yaml file)
             if verbose:
                 print('flag_permanent_settings is set to True')
-            dictonary_settings = self.data_analysis.PermanentSettings.__dict__
+            settings_dict = self.data_analysis.PermanentSettings.__dict__
         else:
             # Read settings from local settings file (yaml file)
-            user_name = os.getlogin()
-            full_path_file_settings = os.path.join(self.relative_path_settings, f"settings.{user_name}.yaml")
+            full_path_file_settings = os.path.join(Path(relative_path_settings).resolve(), settings_file)
             if verbose:
                 print('flag_permanent_settings is set to False')
                 print('user_name:               {}'.format(user_name))
-                print('relative_path_settings:  {}'.format(self.relative_path_settings))
+                print('relative_path_settings:  {}'.format(relative_path_settings))
                 print('full_path_file_settings: {}'.format(full_path_file_settings))
             if not os.path.isfile(full_path_file_settings):
                 raise Exception('Local setting file {} not found. This file must be provided when flag_permanent_settings is set to False.'.format(full_path_file_settings))
-            with open(full_path_file_settings, 'r') as stream:
-                dictonary_settings = yaml.safe_load(stream)
+            settings_dict = yaml_to_data(full_path_file_settings)
 
         # Assign the keys read either from permanent-settings or local-settings
         for name, _ in self.settings.__dict__.items():
-            if name in dictonary_settings:
-                value = dictonary_settings[name]
+            if name in settings_dict:
+                value = settings_dict[name]
                 self.setAttribute(self.settings, name, value)
                 if verbose: print('{} : {}. Added.'.format(name, value))
             else:
                 if verbose: print('{}: not found in the settings. Skipped.'.format(name))
 
         # Dump read keys to temporary settings file locally
-        user_name = os.getlogin()
-        name_file_settings = f"settings.{user_name}.yaml"
-        path_temp_file_settings = Path(os.path.join('', name_file_settings)).resolve()
-        with open(path_temp_file_settings, 'w') as yaml_file:
-            yaml.dump(dictonary_settings, yaml_file, default_flow_style=False, sort_keys=False)
+        path_temp_file_settings = Path(os.path.join('', settings_file)).resolve()
+        dict_to_yaml(settings_dict, path_temp_file_settings)
         if verbose: print('File {} was saved locally.'.format(path_temp_file_settings))
 
     def write_analysis_file(self, path_output_file: str):
@@ -183,15 +162,12 @@ class AnalysisSTEAM:
         :return: None
         """
 
-        verbose = self.verbose
-
         # Make sure the target folder exists
-        make_folder_if_not_existing(os.path.dirname(path_output_file), verbose=verbose)
+        make_folder_if_not_existing(os.path.dirname(path_output_file), verbose=self.verbose)
 
         # Write the STEAM analysis data to a yaml file
-        all_data_dict = {**self.data_analysis.dict()}
-        dict_to_yaml(all_data_dict, path_output_file, list_exceptions=['AnalysisStepSequence', 'variables_to_change'])
-        if verbose: print(f'File {path_output_file} saved.')
+        dict_to_yaml({**self.data_analysis.dict()}, path_output_file, list_exceptions=['AnalysisStepSequence', 'variables_to_change'])
+        if self.verbose: print(f'File {path_output_file} saved.')
 
     def run_analysis(self, verbose: bool = None):
         """
@@ -255,14 +231,14 @@ class AnalysisSTEAM:
         else:
             file_model_data = os.path.join(self.library_path, 'modelData_' + step.file_model_data + '.yaml')
 
-        case_model      = step.case_model
-        software        = step.software  # remember this is a list, not a string
+        case_model = step.case_model
+        software = step.software  # remember this is a list, not a string
         verbose_of_step = step.verbose
-        flag_build      = step.flag_build
-        flag_dump_all   = step.flag_dump_all
-        flag_plot_all   = step.flag_plot_all
-        flag_json       = step.flag_json
-        output_folder   = self.output_path
+        flag_build = step.flag_build
+        flag_dump_all = step.flag_dump_all
+        flag_plot_all = step.flag_plot_all
+        flag_json = step.flag_json
+        output_folder = self.output_path
         relative_path_settings = ''
 
         # Build the model
@@ -274,8 +250,7 @@ class AnalysisSTEAM:
         # Build simulation file
         if step.simulation_number:
             if 'FiQuS' in step.software:
-                self.setup_sim_FiQuS(magnet_type=BM.model_data.GeneralParameters.magnet_type, simulation_name=step.simulation_name,
-                                     simulation_type=step.type, sim_number=step.simulation_number)
+                self.setup_sim_FiQuS(simulation_name=step.simulation_name, simulation_type=step.type, sim_number=step.simulation_number, magnet_name=BM.model_data.GeneralParameters.magnet_name, magnet_type=BM.model_data.GeneralParameters.magnet_type)
             if 'LEDET' in step.software:
                 flag_yaml = True  # Hard-coded for the moment
                 self.setup_sim_LEDET(simulation_name=step.simulation_name, sim_number=step.simulation_number, flag_yaml=flag_yaml, flag_json=flag_json)
@@ -283,6 +258,8 @@ class AnalysisSTEAM:
                 self.setup_sim_PyBBQ(simulation_name=step.simulation_name, sim_number=step.simulation_number)
             if 'PSPICE' in step.software:
                 self.setup_sim_PSPICE(simulation_name=step.simulation_name, sim_number=step.simulation_number)
+            if 'XYCE' in step.software:
+                self.setup_sim_XYCE(simulation_name=step.simulation_name, sim_number=step.simulation_number)
 
         # Add the reference to the model in the dictionary
         self.list_models[step.model_name] = BM
@@ -358,8 +335,8 @@ class AnalysisSTEAM:
                 simulation_number = step.simulation_numbers[v]
                 if 'FiQuS' in step.software:
                     BM.buildFiQuS()
-                    self.setup_sim_FiQuS(magnet_type=BM.model_data.GeneralParameters.magnet_type, simulation_name=step.simulation_name,
-                                         model_name=step.model_name, simulation_type=step.type, sim_number=simulation_number)
+                    #self.setup_sim_FiQuS(magnet_type=BM.model_data.GeneralParameters.magnet_type, simulation_name=step.simulation_name, model_name=step.model_name, simulation_type=step.type, sim_number=simulation_number)
+                    self.setup_sim_FiQuS(simulation_name=step.simulation_name, simulation_type=step.type, sim_number=simulation_number, magnet_name=BM.model_data.GeneralParameters.magnet_name, magnet_type=BM.model_data.GeneralParameters.magnet_type)
                 if 'LEDET' in step.software:
                     flag_json = BM.flag_json
                     flag_yaml = True  # Hard-coded for the moment
@@ -371,6 +348,9 @@ class AnalysisSTEAM:
                 if 'PSPICE' in step.software:
                     BM.buildPSPICE()
                     self.setup_sim_PSPICE(simulation_name=step.simulation_name, sim_number=simulation_number)
+                if 'XYCE' in step.software:
+                    BM.buildXYCE()
+                    self.setup_sim_XYCE(simulation_name=step.simulation_name, sim_number=simulation_number)
 
     def get_attribute_model(self, case_model: str, builder_model: BuilderModel, name_variable: str, idx_conductor: int = None):
         """
@@ -402,7 +382,7 @@ class AnalysisSTEAM:
 
     def set_attribute_model(self, case_model: str, builder_model: BuilderModel, name_variable: str, value_variable: Union[int, float, str], idx_conductor: int = None):
         """
-        Helper function used to set a key of the model data to a certaub value.
+        Helper function used to set a key of the model data to a certain value.
         Depending on the model type (circuit, magnet, conductor), the data structure to access is different.
         Also, there is a special case when the variable to change is a sub-key of the Conductors key. In such a case, an additional parameter idx_conductor must be defined (see below).
         :param case_model: Model type
@@ -436,7 +416,7 @@ class AnalysisSTEAM:
         if step.model_name not in self.list_models:
             raise Exception('Name of the model to modify ({}) does not correspond to any of the defined models.'.format(step.model_name))
         len_variables_to_change = len(step.variables_to_change)
-        len_variables_value     = len(step.variables_value)
+        len_variables_value = len(step.variables_value)
         if not len_variables_to_change == len_variables_value:
             raise Exception('The length of variables_to_change and variables_value must be the same, but they are {} and {} instead.'.format(len_variables_to_change, len_variables_value))
 
@@ -447,7 +427,7 @@ class AnalysisSTEAM:
             next_step.model_name = step.model_name
             next_step.variable_to_change = variable_to_change
             next_step.variable_value = step.variables_value[v]
-            if v+1 == len_variables_to_change:  # If this is the last variable to change, import new_model_name and simulation_numbers from the step
+            if v + 1 == len_variables_to_change:  # If this is the last variable to change, import new_model_name and simulation_numbers from the step
                 next_step.new_model_name = step.new_model_name
                 next_step.simulation_numbers = step.simulation_numbers
             else:  # else, set new_model_name and simulation_numbers to empty lists to avoid making models/simulations for intermediate changes
@@ -471,6 +451,7 @@ class AnalysisSTEAM:
 
     def step_postprocess(self, step, verbose: bool = False):
         if verbose: print('postprocessing')
+        print('Not implemented yet.')
         pass
 
     def step_setup_folder(self, step, verbose: bool = False):
@@ -485,7 +466,14 @@ class AnalysisSTEAM:
             if verbose:
                 print('Set up folder of model {} for {}.'.format(simulation_name, software))
 
-            if 'LEDET' in software:
+            if 'FiQuS' in software:
+                # make top level output folder
+                make_folder_if_not_existing(self.settings.local_FiQuS_folder, verbose=verbose)
+
+                # make simulation name folder inside top level folder
+                make_folder_if_not_existing(os.path.join(self.settings.local_FiQuS_folder, step.simulation_name))
+
+            elif 'LEDET' in software:
                 local_LEDET_folder = Path(self.settings.local_LEDET_folder)
                 # Make magnet input folder and its subfolders
                 make_folder_if_not_existing(Path(local_LEDET_folder / simulation_name / 'Input').resolve(), verbose=verbose)
@@ -496,8 +484,8 @@ class AnalysisSTEAM:
                 # Copy csv files from the output folder
                 list_csv_files = [entry for entry in os.listdir(self.output_path) if (simulation_name in entry) and ('.csv' in entry)]
                 for csv_file in list_csv_files:
-                    file_to_copy = os.path.join(self.output_path,                                               csv_file)
-                    file_copied  = os.path.join(Path(local_LEDET_folder / simulation_name / 'Input').resolve(), csv_file)
+                    file_to_copy = os.path.join(self.output_path, csv_file)
+                    file_copied = os.path.join(Path(local_LEDET_folder / simulation_name / 'Input').resolve(), csv_file)
                     shutil.copyfile(file_to_copy, file_copied)
                     if verbose: print('Csv file {} copied to {}.'.format(file_to_copy, file_copied))
 
@@ -508,8 +496,8 @@ class AnalysisSTEAM:
                 # Copy field-map files from the output folder
                 list_field_maps = [entry for entry in os.listdir(self.output_path) if (simulation_name in entry) and ('.map2d' in entry)]
                 for field_map in list_field_maps:
-                    file_to_copy = os.path.join(self.output_path,  field_map)
-                    file_copied  = os.path.join(field_maps_folder, field_map)
+                    file_to_copy = os.path.join(self.output_path, field_map)
+                    file_copied = os.path.join(field_maps_folder, field_map)
                     shutil.copyfile(file_to_copy, file_copied)
                     if verbose: ('Field map file {} copied to {}.'.format(file_to_copy, file_copied))
 
@@ -530,8 +518,38 @@ class AnalysisSTEAM:
                 # Copy stl files from the output folder
                 list_stl_files = [entry for entry in os.listdir(self.output_path) if (simulation_name in entry) and ('.stl' in entry)]
                 for stl_file in list_stl_files:
-                    file_to_copy = os.path.join(self.output_path,   stl_file)
-                    file_copied  = os.path.join(local_model_folder, stl_file)
+                    file_to_copy = os.path.join(self.output_path, stl_file)
+                    file_copied = os.path.join(local_model_folder, stl_file)
+                    shutil.copyfile(file_to_copy, file_copied)
+                    if verbose: print('Stl file {} copied to {}.'.format(file_to_copy, file_copied))
+
+            elif 'XYCE' in software:
+                local_XYCE_folder = Path(self.settings.local_XYCE_folder)
+                local_model_folder = str(Path(local_XYCE_folder / simulation_name).resolve())
+                # Make circuit input folder
+                make_folder_if_not_existing(local_model_folder, verbose=verbose)
+
+                # Copy lib files from the output folder
+                list_lib_files = [entry for entry in os.listdir(self.output_path) if
+                                  (simulation_name in entry) and ('.lib' in entry)]
+                for lib_file in list_lib_files:
+                    file_to_copy = os.path.join(self.output_path, lib_file)
+                    file_copied = os.path.join(local_model_folder, lib_file)
+                    shutil.copyfile(file_to_copy, file_copied)
+                    if verbose: print('Lib file {} copied to {}.'.format(file_to_copy, file_copied))
+
+                # Copy stl files from the output folder
+                stl_path = os.path.join(self.output_path, 'Stimulus')
+                list_stl_files = [entry for entry in os.listdir(stl_path) if
+                                  (simulation_name in entry) and ('.csv' in entry)]
+                stl_path_new = os.path.join(local_model_folder, 'Stimulus')
+                if os.path.exists(stl_path_new):
+                    shutil.rmtree(stl_path_new)
+                os.mkdir(stl_path_new)
+
+                for stl_file in list_stl_files:
+                    file_to_copy = os.path.join(self.output_path, stl_file)
+                    file_copied = os.path.join(stl_path_new, stl_file)
                     shutil.copyfile(file_to_copy, file_copied)
                     if verbose: print('Stl file {} copied to {}.'.format(file_to_copy, file_copied))
 
@@ -544,8 +562,8 @@ class AnalysisSTEAM:
         """
         # Unpack
         full_path_aux_file = Path(step.full_path_aux_file).resolve()
-        new_file_name      = step.new_file_name
-        output_path        = self.output_path
+        new_file_name = step.new_file_name
+        output_path = self.output_path
 
         # If no new name is provided, use the old file name
         if new_file_name == None:
@@ -570,6 +588,8 @@ class AnalysisSTEAM:
                     self.setup_sim_PyBBQ(simulation_name=step.simulation_name, sim_number=simulation_number)
                 if 'PSPICE' in step.software:
                     self.setup_sim_PSPICE(simulation_name=step.simulation_name, sim_number=simulation_number)
+                if 'PSPICE' in step.software:
+                    self.setup_sim_XYCE(simulation_name=step.simulation_name, sim_number=simulation_number)
 
     def copy_file_to_target(self, step, verbose: bool = False):
         """
@@ -577,7 +597,7 @@ class AnalysisSTEAM:
         """
         # Unpack
         full_path_file_to_copy = Path(step.full_path_file_to_copy).resolve()
-        full_path_file_target  = Path(step.full_path_file_target).resolve()
+        full_path_file_target = Path(step.full_path_file_target).resolve()
 
         # Make sure the target folder exists
         make_folder_if_not_existing(os.path.dirname(full_path_file_target), verbose=verbose)
@@ -586,7 +606,6 @@ class AnalysisSTEAM:
         shutil.copyfile(full_path_file_to_copy, full_path_file_target)
         if verbose: print(f'File {full_path_file_to_copy} was copied to {full_path_file_target}.')
 
-
     def run_custom_py_function(self, step, verbose: bool = False):
         """
             Run a custom Python function with given arguments
@@ -659,6 +678,7 @@ class AnalysisSTEAM:
 
         # Unpack variables
         viewer_name = step.viewer_name
+        metrics_name = step.metrics_name
         metrics_to_calculate = step.metrics_to_calculate
         variables_to_analyze = step.variables_to_analyze
         # Note: Avoid unpacking "list_viewers = self.list_viewers" since the variable usually has large size
@@ -669,15 +689,23 @@ class AnalysisSTEAM:
         # if len(metrics_to_calculate) != len(variables_to_analyze):
         #     raise Exception(f'The lengths of the lists metrics_to_calculate and variables_to_analyze must match, but are {len(metrics_to_calculate)} and {len(variables_to_analyze)} instead.')
 
+        # If the Analysis object contains a metrics set with the selected metrics_name, retrieve it: the new metrics entries will be appended to it
+        if metrics_name in self.list_metrics:
+            current_list_output_metrics = self.list_metrics[metrics_name]
+        else:
+            # If not, make a new metrics set
+            current_list_output_metrics = {}
+
         # Loop through all events listed in the selected Viewer object
         for event_id in self.list_viewers[viewer_name].list_events:
-            event_label        = self.list_viewers[viewer_name].dict_events['Event label'][event_id-1]
+            event_label = self.list_viewers[viewer_name].dict_events['Event label'][event_id - 1]
             if verbose: print(f'Event #{event_id}: "{event_label}".')
+            current_list_output_metrics[event_label] = {}
 
             # For each selected pair of variables to analyze, calculate metrics
             for pair_var in variables_to_analyze:
                 var_to_analyze = pair_var[0]
-                var_reference  = pair_var[1]
+                var_reference = pair_var[1]
 
                 # Check that the selected signal to analyze and its reference signal (if they are defined) exist in the current event
                 if len(var_to_analyze) > 0:
@@ -719,42 +747,40 @@ class AnalysisSTEAM:
                 if verbose: print(f'Viewer "{viewer_name}": Event "{event_label}": Metrics calculated using signals "{var_to_analyze}" and "{var_reference}".')
 
                 # Calculate the metrics
-                # output_metric = Metrics(
+                # output_metric = PostprocsMetrics(
                 #     metrics_to_calculate=metrics_to_calculate,
-                #     x_value=y_var_to_analyze,
+                #     x_value=x_var_to_analyze,
                 #     y_value=y_var_to_analyze,
                 #     x_ref=x_var_reference,
                 #     y_ref=y_var_reference,
                 #     flag_run=True)
+                output_metric = PostprocsMetrics(metrics_to_do=metrics_to_calculate, var_to_interpolate=y_var_to_analyze,
+                                                 var_to_interpolate_ref=y_var_reference, time_vector=x_var_to_analyze,
+                                                 time_vector_ref=x_var_reference)
+                current_list_output_metrics[event_label][var_to_analyze] = output_metric.metrics_result
 
+        # Add the reference to the Viewer object in the dictionary
+        self.list_metrics[metrics_name] = current_list_output_metrics
 
-    def setup_sim_FiQuS(self, simulation_name, simulation_type, sim_number, model_name=None, magnet_type=None):
+        return current_list_output_metrics
+
+    def setup_sim_FiQuS(self, simulation_name, simulation_type, sim_number, magnet_name=None, magnet_type=None):
         """
         Set up a FiQuS simulation by copying the last file generated by BuilderModel to the output folder and to the
         local FiQuS working folder.
         The original file is then deleted.
         """
-        def _find_original_model(name):
-            src = None
-            for k, s in self.data_analysis.AnalysisStepDefinition.items():
-                if s.type == 'MakeModel':
-                    if s.model_name == name:
-                        src = os.path.join(os.path.dirname(local_model_folder), str(s.simulation_number), simulation_name)
-            return src
 
-        # Unpack
-        output_folder = self.output_path
-        local_FiQuS_folder = Path(self.settings.local_FiQuS_folder)
 
         # Add simulation number to the list
         self.list_sims.append(sim_number)
 
         # Make simulation folder
-        local_model_folder = os.path.join(local_FiQuS_folder, 'Analysis_' + simulation_name, str(sim_number))
+        local_model_folder = os.path.join(self.settings.local_FiQuS_folder, simulation_name + '_' + str(sim_number))
         make_folder_if_not_existing(local_model_folder)
 
         # Copy simulation file
-        file_name_temp = os.path.join(output_folder, simulation_name)
+        file_name_temp = os.path.join(self.output_path, magnet_name)
         yaml_temp = os.path.join(file_name_temp + '_FiQuS.yaml')
         file_name_local = os.path.join(local_model_folder, simulation_name)
         yaml_local = os.path.join(file_name_local + '.yaml')
@@ -768,16 +794,6 @@ class AnalysisSTEAM:
                 set_local = os.path.join(file_name_local + '.set')
                 shutil.copyfile(geo_temp, geo_local)
                 shutil.copyfile(set_temp, set_local)
-            else:
-                source = _find_original_model(model_name)
-                while not source:
-                    for key, step in self.data_analysis.AnalysisStepDefinition.items():
-                        if step.type in ['ModifyModel', 'ModifyModelMultipleVariables']:
-                            if model_name in step.new_model_name:
-                                new_name = step.model_name
-                                source = _find_original_model(new_name)
-                os.symlink(source + '.geom', file_name_local + '.geom')
-                os.symlink(source + '.set', file_name_local + '.set')
 
         if self.verbose: print('Simulation files {} generated.'.format(file_name_temp))
         if self.verbose: print('Simulation files {} copied.'.format(file_name_local))
@@ -791,7 +807,7 @@ class AnalysisSTEAM:
         """
 
         # Unpack
-        output_folder      = self.output_path
+        output_folder = self.output_path
         local_LEDET_folder = self.settings.local_LEDET_folder
 
         # Add simulation number to the list
@@ -805,9 +821,9 @@ class AnalysisSTEAM:
             list_suffix.append('.json')
 
         for suffix in list_suffix:
-            file_name_temp   = os.path.join(output_folder, simulation_name + suffix)
+            file_name_temp = os.path.join(output_folder, simulation_name + suffix)
             file_name_output = os.path.join(output_folder, simulation_name + '_' + str(sim_number) + suffix)
-            file_name_local  = os.path.join(local_LEDET_folder, simulation_name, 'Input', simulation_name + '_' + str(sim_number) + suffix)
+            file_name_local = os.path.join(local_LEDET_folder, simulation_name, 'Input', simulation_name + '_' + str(sim_number) + suffix)
 
             shutil.copyfile(file_name_temp, file_name_output)
             if self.verbose: print('Simulation file {} generated.'.format(file_name_output))
@@ -818,7 +834,6 @@ class AnalysisSTEAM:
             # os.remove(file_name_temp)
             # print('Temporary file {} deleted.'.format(file_name_temp))
 
-
     def setup_sim_PSPICE(self, simulation_name, sim_number):
         """
         Set up a PSPICE simulation by copying the last file generated by BuilderModel to the output folder and to the
@@ -860,11 +875,69 @@ class AnalysisSTEAM:
         # Copy stl files from the output folder
         list_stl_files = [entry for entry in os.listdir(output_folder) if (simulation_name in entry) and ('.stl' in entry)]
         for stl_file in list_stl_files:
-            file_to_copy = os.path.join(output_folder,   stl_file)
-            file_copied  = os.path.join(local_model_folder, stl_file)
+            file_to_copy = os.path.join(output_folder, stl_file)
+            file_copied = os.path.join(local_model_folder, stl_file)
             shutil.copyfile(file_to_copy, file_copied)
             if self.verbose: print('Stl file {} copied to {}.'.format(file_to_copy, file_copied))
 
+    def setup_sim_XYCE(self, simulation_name, sim_number):
+        """
+        Set up a PSPICE simulation by copying the last file generated by BuilderModel to the output folder and to the
+        local PSPICE working folder.
+        The simulation netlist and auxiliary files are copied in a new numbered subfoldered.
+        The original file is then deleted.
+        """
+
+        # Unpack
+        output_folder = self.output_path
+        local_XYCE_folder = Path(self.settings.local_XYCE_folder)
+
+        # Add simulation number to the list
+        self.list_sims.append(sim_number)
+
+        # Make simulation folder
+        local_model_folder = os.path.join(local_XYCE_folder, simulation_name, str(sim_number))
+        make_folder_if_not_existing(local_model_folder)
+
+        # Copy simulation file
+        file_name_temp = os.path.join(output_folder, simulation_name + '.cir')
+        file_name_local = os.path.join(local_model_folder, simulation_name + '.cir')
+        if self.verbose: print('Simulation file {} generated.'.format(file_name_temp))
+
+        shutil.copyfile(file_name_temp, file_name_local)
+        if self.verbose: print('Simulation file {} copied.'.format(file_name_local))
+
+        # os.remove(file_name_temp)
+        # print('Temporary file {} deleted.'.format(file_name_temp))
+
+        local_XYCE_folder = Path(self.settings.local_XYCE_folder)
+        local_model_folder = str(Path(local_XYCE_folder / simulation_name).resolve())
+        # Make circuit input folder
+        make_folder_if_not_existing(local_model_folder, verbose=self.verbose)
+
+        # Copy lib files from the output folder
+        list_lib_files = [entry for entry in os.listdir(self.output_path) if
+                          (simulation_name in entry) and ('.lib' in entry)]
+        for lib_file in list_lib_files:
+            file_to_copy = os.path.join(self.output_path, lib_file)
+            file_copied = os.path.join(local_model_folder, lib_file)
+            shutil.copyfile(file_to_copy, file_copied)
+            if self.verbose: print('Lib file {} copied to {}.'.format(file_to_copy, file_copied))
+
+        # Copy stl files from the output folder
+        stl_path = os.path.join(self.output_path, 'Stimulus')
+        stl_path_new = os.path.join(local_model_folder, 'Stimulus')
+        if os.path.exists(stl_path_new):
+            shutil.rmtree(stl_path_new)
+        os.mkdir(stl_path_new)
+
+        list_stl_files = [entry for entry in os.listdir(stl_path) if
+                          (simulation_name in entry) and ('.csv' in entry)]
+        for stl_file in list_stl_files:
+            file_to_copy = os.path.join(self.output_path, stl_file)
+            file_copied = os.path.join(stl_path_new, stl_file)
+            shutil.copyfile(file_to_copy, file_copied)
+            if self.verbose: print('Stl file {} copied to {}.'.format(file_to_copy, file_copied))
 
     def setup_sim_PyBBQ(self, simulation_name, sim_number):
         """
@@ -895,19 +968,16 @@ class AnalysisSTEAM:
         # os.remove(file_name_temp)
         # print('Temporary file {} deleted.'.format(file_name_temp))
 
-
     def run_sim(self, software: str, simulation_name: str, sim_number: int, simFileType: str = None, verbose: bool = False):
         """
         Run selected simulation.
         The function applies a different logic for each simulation software.
         """
         if software == 'FiQuS':
-            local_analysis_folder = 'Analysis_' + simulation_name
-            local_model_folder_input = os.path.join(self.settings.local_FiQuS_folder, local_analysis_folder, str(sim_number))
+            local_analysis_folder = simulation_name + '_' + str(sim_number)
             dFiQuS = DriverFiQuS(FiQuS_path=self.settings.FiQuS_path, path_folder_FiQuS=self.settings.local_FiQuS_folder,
-                                 path_folder_FiQuS_input=local_model_folder_input, verbose=verbose)
-            dFiQuS.run_FiQuS(sim_file_name=simulation_name,
-                             output_directory=os.path.join(local_analysis_folder, simulation_name))
+                                 path_folder_FiQuS_input=os.path.join(self.settings.local_FiQuS_folder, local_analysis_folder), verbose=verbose)
+            dFiQuS.run_FiQuS(sim_file_name=simulation_name, output_directory=local_analysis_folder)
         elif software == 'LEDET':
             dLEDET = DriverLEDET(path_exe=self.settings.LEDET_path, path_folder_LEDET=self.settings.local_LEDET_folder, verbose=verbose)
             dLEDET.run_LEDET(simulation_name, str(sim_number), simFileType=simFileType)
@@ -920,6 +990,9 @@ class AnalysisSTEAM:
             local_model_folder = Path(Path(self.settings.local_PSPICE_folder) / simulation_name / str(sim_number)).resolve()
             dPSPICE = DriverPSPICE(path_exe=self.settings.PSPICE_path, path_folder_PSPICE=local_model_folder, verbose=verbose)
             dPSPICE.run_PSPICE(simulation_name, suffix='')
+        elif software == 'XYCE':
+            local_model_folder = Path(Path(self.settings.local_XYCE_folder) / simulation_name / str(sim_number)).resolve()
+            dXYCE = DriverXYCE(path_exe=self.settings.XYCE_path, path_folder_XYCE=local_model_folder, verbose=verbose)
+            dXYCE.run_XYCE(simulation_name, suffix='')
         else:
             raise Exception(f'Software {software} not supported for automated running.')
-
