@@ -14,11 +14,12 @@ from steam_sdk.builders.BuilderSIGMA import BuilderSIGMA
 from steam_sdk.parsers.ParserFiQuS import ParserFiQuS
 from steam_sdk.parsers.ParserLEDET import ParserLEDET, copy_modified_map2d_ribbon_cable, copy_map2d
 from steam_sdk.parsers.ParserPSPICE import ParserPSPICE
+from steam_sdk.parsers.ParserXYCE import ParserXYCE
 from steam_sdk.parsers.ParserProteCCT import ParserProteCCT
 from steam_sdk.parsers.ParserProtePyBBQ import ParserPyBBQ
 from steam_sdk.parsers.ParserRoxie import ParserRoxie
 from steam_sdk.parsers.ParserMap2d import getParametersFromMap2d
-from steam_sdk.parsers.ParserYAML import dict_to_yaml, yaml_to_dict, yaml_to_data_class
+from steam_sdk.parsers.ParserYAML import dict_to_yaml, yaml_to_data
 from steam_sdk.plotters import PlotterRoxie
 from steam_sdk.plotters.PlotterModel import PlotterModel
 
@@ -127,6 +128,7 @@ class BuilderModel:
                 # Build circuit model
                 for s in self.software:
                     if s == 'PSPICE': self.buildPSPICE()
+                    elif s == 'XYCE': self.buildXYCE()
 
         ### Case of a conductor model
         elif self.case_model == 'conductor':
@@ -203,7 +205,7 @@ class BuilderModel:
         settings_file = Path.joinpath(self.path_settings, f"settings.{user_name}.yaml")
         if not Path.exists(settings_file):
             settings_file = Path.joinpath(Path(os.getcwd()).parent, "settings.SYSTEM.yaml")
-        self.settings_dict = yaml_to_dict(settings_file)
+        self.settings_dict = yaml_to_data(settings_file)
 
         # Display defined paths
         if self.verbose:
@@ -227,14 +229,20 @@ class BuilderModel:
 
         if self.case_model == 'magnet':
             # Load yaml keys into DataModelMagnet dataclass
-            self.model_data = yaml_to_data_class(self.file_model_data, DataModelMagnet)
+            self.model_data = yaml_to_data(self.file_model_data, DataModelMagnet)
         elif self.case_model == 'circuit':
-            Parser = ParserPSPICE(self.circuit_data)
-            Parser.readFromYaml(self.file_model_data)
-            self.circuit_data = Parser.circuit_data
+            for s in self.software:
+                if s=='PSPICE':
+                    Parser = ParserPSPICE(self.circuit_data)
+                    Parser.readFromYaml(self.file_model_data)
+                    self.circuit_data = Parser.circuit_data
+                elif s=='XYCE':
+                    Parser = ParserXYCE(self.circuit_data)
+                    Parser.readFromYaml(self.file_model_data)
+                    self.circuit_data = Parser.circuit_data
         elif self.case_model == 'conductor':
             # Load yaml keys into DataModelConductor dataclass
-            self.conductor_data = yaml_to_data_class(self.file_model_data, DataModelConductor)
+            self.conductor_data = yaml_to_data(self.file_model_data, DataModelConductor)
         else:
             raise ValueError('results: case_model must be one of {}.'.format(['magnet', 'circuit', 'conductor']))
 
@@ -459,8 +467,6 @@ class BuilderModel:
         """
             Build a PSPICE circuit netlist model
         """
-
-
         # Write output .cir file
         parser_pspice = ParserPSPICE(circuit_data=self.circuit_data, path_input=self.path_magnet,
                                      output_path=self.output_path)
@@ -470,6 +476,19 @@ class BuilderModel:
         # Copy additional files
         parser_pspice.copy_additional_files()
 
+
+    def buildXYCE(self):
+        """
+            Build a PSPICE circuit netlist model
+        """
+        # Write output .cir file
+        parser_xyce = ParserXYCE(circuit_data=self.circuit_data, path_input=self.path_magnet,output_path=self.output_path)
+        nameFileXYCE = os.path.join(self.output_path, self.circuit_data.GeneralParameters.circuit_name + '.cir')  # full path of the PSPICE netlist to write
+        parser_xyce.write2XYCE(nameFileXYCE, verbose=self.verbose)
+        parser_xyce.copy_additional_files(flag_translate=True, verbose=self.verbose)
+        parser_xyce.resolve_paths_netlist(nameFileXYCE)
+
+
     def buildPyBBQ(self):
         """
             Building a PyBBQ model
