@@ -146,7 +146,10 @@ class PhysicsPyBBQ(BaseModel):
     aKap: float = None
     BBackground: float = None
     BPerI: float = None
+    Heating_mode: str = None
     Heating_nodes: List[int] = None
+    Heating_time: float = None
+    Heating_time_constant: float = None
     IDesign: float = None
     Jc_4K_5T_NbTi: float = None
     jointLength: float = None
