@@ -328,7 +328,7 @@ class MultipolePostProc(BaseModel):
         Level 2: Class for FiQuS Multipole
     """
     compare_to_ROXIE: str = None
-    plot_all: bool = None
+    plot_all: str = None
     variables: List[str] = None  # Name of variables to post-process, like "b" for magnetic flux density
     volumes: List[str] = None  # Name of domains to post-process, like "powered"
     file_exts: List[str] = None  # Name of file extensions to output to, like "pos"
@@ -377,7 +377,6 @@ class Former_g(BaseModel):  # Geometry related formers _inputs
         Level 2: Class for FiQuS CCT Options
     """
     names: List[str] = None  # name to use in gmsh and getdp
-    r_outs: List[float] = None  # outer radius
     z_mins: List[float] = None  # extend of former  in negative z direction
     z_maxs: List[float] = None  # extend of former in positive z direction
 
@@ -437,7 +436,6 @@ class SolveCCT(BaseModel):
     formers: Former_s = Former_s()  # former solution time _inputs
     air: Air_s = Air_s()  # air solution time _inputs
     pro_template: str = None  # file name of .pro template file
-    getdp_exe_path: str = None  # absolute path to GetDP executable
     variables: List[str] = None  # Name of variable to post-process by GetDP, like B for magnetic flux density
     volumes: List[str] = None  # Name of volume to post-process by GetDP, line Winding_1
     file_exts: List[str] = None  # Name of file extensions to post-process by GetDP, like .pos
