@@ -6,7 +6,7 @@ from ruamel import yaml
 
 from steam_sdk.data.DataAnalysis import ModifyModel, AddAuxiliaryFile
 from steam_sdk.data.DataAnalysis import DataAnalysis
-from steam_sdk.parsers.ParserYAML import yaml_to_data_class
+from steam_sdk.parsers.ParserYAML import yaml_to_data
 
 
 class DriverAnalysis:
@@ -17,7 +17,7 @@ class DriverAnalysis:
         #self.input_variables = self.Builder.DAKOTA_data.DAKOTA_analysis.variables
         #self.global_run_counter = 0
 
-        self.analysis_data = yaml_to_data_class(analysis_yaml_path, DataAnalysis)
+        self.analysis_data = yaml_to_data(analysis_yaml_path, DataAnalysis)
         path_dakota_python = os.path.join(Path(self.analysis_data.PermanentSettings.Dakota_path).parent.parent, 'share\dakota\Python')
         print('path_dakota_python:        {}'.format(path_dakota_python))
         sys.path.insert(0, path_dakota_python)
