@@ -19,8 +19,8 @@ class DriverDakota:
         """
         os.chdir(self.dakota_output_folder)
         with open('driver_link.py', 'w') as f:
-            f.write('from steam_sdk.drivers.DriverAnalysis import DriverAnalysis')
-            f.write(f'\nda = DriverAnalysis(r"{self.analysis_yaml_path}")')
+            f.write('from steam_sdk.parsims.ParsimDakota import ParsimDakota')
+            f.write(f'\nda = ParsimDakota(r"{self.analysis_yaml_path}")')
             f.write('\nda.drive()')
 
         subprocess.call([self.Dakota_path, '-i', input_file_path])
