@@ -2,8 +2,24 @@ import os
 import numpy as np
 import pandas as pd
 from pathlib import Path
+import ruamel.yaml
+import csv
+from steam_sdk.parsers.ParserYAML import dict_to_yaml
 
 
+def find_by_position(filename, idx, value):
+    with open(filename) as f:
+        reader = csv.reader(f)
+        row = next((item for item in reader if item[idx] == value), None)
+
+    return row
+
+
+def find_column_list(filename):
+    with open(filename) as f:
+        reader = csv.reader(f)
+        row1 = next(reader)
+    return row1
 def getSpecificSignalCSV(path_sim, model_name, simNumber, list_Signals):
 
     simulationSignalsToPlot = pd.DataFrame()
@@ -40,37 +56,49 @@ def get_signals_from_csv(full_name_file: str, list_signals):
     all_signals_df.columns = all_signals_df.columns.str.replace(' ', '')  # eliminate whitespaces from the column names
     list_signals = [x.replace(' ', '') for x in list_signals]             # eliminate whitespaces from the signal names as well
     return all_signals_df[list_signals]
-
-
-def load_parameters_from_csv(full_name_file: str, name_row: str, path_model_library: str, case_model: str, flag_write_file: bool):
+def check_global_parameters_from_csv_table(filename: str, circuit_name: str,steam_models_path: str,case_model: str,flag_write_file:bool=False ):
     '''
-    Reads a csv file and returns a dataframe with the selected signals
-    :param full_name_file: full path to the csv file with the parameters
-    :param name_row: this is a string indentifying the correct row in the csv file
-    :param path_model_library: path of the steam_models folder where the items are located
-    :param case_model: conductor, magnet, or circuit
-    :return: 
+        Reads a csv file and circuit name and checks that the global parameters for the circuit are consistent with the csv file. If not, changes are made in the circuit parameters
+        :param filename: full path to the csv file
+        :param circuit_name: circuit name eg. RCS.A12B1
+        :param steam_models_path: path to steam models
+        :param case_model: folder within steam models eg. circuit or magnet
+        :param flag_write_file: whether changes are made or not
+        :param out_file: changed yaml file
+
+        :return: out_file
     '''
+    circuit_param=[]
+    param_names=[]
 
-    # Load csv file as dataframe
-    
-    # Identify the correct model input yaml file (use path_model_library and case_model and the circuit name from the dataframe)
-    model_name = 'RCS' # this needs to be read from the input csv file 'RCS'
-    model_file_name = f'modelData_{model_name}.yaml'
-    full_path_file = Path(os.path.join(path_model_library, case_model + 's', model_name, 'input', model_file_name)).resolve()
-    
-    # Import model yaml file as a dict
-    
-    # Loop through the columns of the dataframe and change the parameters that correspond to dataframe columns
-    
-    # Optionally write an output file (don't overwrite the model yaml file)
-
-    # return: ?
-    
-    pass
-    
+    circuit_param= find_by_position(filename, 0, circuit_name)
+    param_names=find_column_list(filename)
+    dict_circuit = {k: v for k, v in zip(param_names, circuit_param)}
+    yaml = ruamel.yaml.YAML()
+    yaml.preserve_quotes = True
+    name = circuit_name.split('.')[0]
+    path_to_model_circuit_data=os.path.join(steam_models_path,case_model+"s",name,"input","modelData_"+name+".yaml")
 
 
+    with open(path_to_model_circuit_data) as fp:
+        data = yaml.load(fp)
+
+        #Write the data before change to an auto.yaml file
+    path_previous_record="previous_global_param_auto.yaml"
+    dict_to_yaml(data, path_previous_record)
+
+    for key, value in dict_circuit.items():
+        if key in data['GlobalParameters']['global_parameters']:
+            if data['GlobalParameters']['global_parameters'][key] != value:
+                print(data['GlobalParameters']['global_parameters'][key], "is different from", value)
+                data['GlobalParameters']['global_parameters'][key] = value
+                flag_write_file=True
+    output_file=os.path.join('output', 'load_parameters_from_csv','modelData_'+circuit_name+".yaml")
+    with open('modelData_'+circuit_name+".yaml", 'w') as f:
+        yaml.dump(data, f)
+    out_file=os.path.join(os.getcwd(),'modelData_'+circuit_name+".yaml")
+    #yaml.dump(data, sys.stdout)
+    return out_file
 # print(get_signals_from_csv('C:/cernbox/steam-models-dev/viewer/RCS/RCS.csv',[]))
 # def gettime_vectorCSV(path_sim, simNumber):
 #
